use std::{env, thread::sleep, time::Duration, error::Error};

use cassandra_cpp::{stmt, Cluster, Session};

pub mod todo_repository;
pub mod user_repository;

/// Repository Result type to use custom errors (abbreviation)
type Result<T> = std::result::Result<T, Box<dyn Error>>;

/// Keyspace name if none is given via ENV: KEYSPACE_NAME
static DEFAULT_KEYSPACE_NAME: &str = "rust_solidjs_cassandra";

/// Waits for the cassandra database to become available -> then returns a session.
pub fn init() -> Session {
    let keyspace_name = env::var("KEYSPACE_NAME").unwrap_or(DEFAULT_KEYSPACE_NAME.to_string());
    // Definitely set it so other modules can use it
    env::set_var("KEYSPACE_NAME", &keyspace_name);

    let mut cluster = Cluster::default();
    cluster.set_contact_points("127.0.0.1").unwrap(); // Panic if not successful
    let mut count = 0;
    let session = loop {
        match cluster.connect() {
            Ok(session) => break session,
            Err(_) => {
                count += 1;
                log::warn!("Could not connect to database! Retrying... ({count})");
                sleep(Duration::new(5, 0));
            },
        }
    };
    let query = stmt!(&format!(
        "CREATE KEYSPACE IF NOT EXISTS {keyspace_name}
             WITH replication = {{'class': 'SimpleStrategy', 'replication_factor': 1}};"
    ));

    session
        .execute(&query)
        .wait()
        .expect("Should create default keyspace if not exists");

    session
}
